;Copyright (C) 2004-2007 John T. Haller of PortableApps.com

;Website: http://PortableApps.com/AudacityPortable

;This software is OSI Certified Open Source Software.
;OSI Certified is a certification mark of the Open Source Initiative.

;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.

;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.

;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

!define FULLNAME "Audacity Portable"
!define NAME "AudacityPortable"
!define APP "Audacity"
!define VER "1.2.4.0"
!define WEBSITE "PortableApps.com/AudacityPortable"
!define DEFAULTEXE "audacity.exe"
!define DEFAULTAPPDIR "audacity"
!define DEFAULTSETTINGSPATH "settings"

;=== Program Details
Name "${FULLNAME}"
OutFile "${NAME}.exe"
Caption "${FULLNAME} | PortableApps.com"
VIProductVersion "${VER}"
VIAddVersionKey ProductName "${FULLNAME}"
VIAddVersionKey Comments "Allows ${APP} to be run from a removable drive.  For additional details, visit ${WEBSITE}"
VIAddVersionKey CompanyName "PortableApps.com (John T. Haller)"
VIAddVersionKey LegalCopyright "John T. Haller"
VIAddVersionKey FileDescription "${FULLNAME}"
VIAddVersionKey FileVersion "${VER}"
VIAddVersionKey ProductVersion "${VER}"
VIAddVersionKey InternalName "${FULLNAME}"
VIAddVersionKey LegalTrademarks "PortableApps.com is a Trademark of Rare Ideas, LLC."
VIAddVersionKey OriginalFilename "${NAME}.exe"
;VIAddVersionKey PrivateBuild ""
;VIAddVersionKey SpecialBuild ""

;=== Runtime Switches
CRCCheck On
WindowIcon Off
SilentInstall Silent
AutoCloseWindow True
RequestExecutionLevel user

;=== Include
!include "Registry.nsh"
!include "GetParameters.nsh"

;=== Program Icon
Icon "${NAME}.ico"

Var PROGRAMDIRECTORY
Var SETTINGSDIRECTORY
Var LAMEDIRECTORY
Var ADDITIONALPARAMETERS
Var REGISTRYDIRECTORY
Var EXECSTRING
Var PROGRAMEXECUTABLE
Var INIPATH
Var DISABLESPLASHSCREEN
Var ISDEFAULTDIRECTORY
Var APPTEMPDIRECTORY
Var TEMPDIREXISTS


Section "Main"
	;=== Find the INI file, if there is one
		IfFileExists "$EXEDIR\${NAME}.ini" "" CheckSubINI
			StrCpy "$INIPATH" "$EXEDIR"
			Goto ReadINI

	CheckSubINI:
		IfFileExists "$EXEDIR\${NAME}\${NAME}.ini" "" NoINI
			StrCpy "$INIPATH" "$EXEDIR\${NAME}"
			Goto ReadINI

	ReadINI:
		;=== Read the parameters from the INI file
		ReadINIStr $0 "$INIPATH\${NAME}.ini" "${NAME}" "${APP}Directory"
		StrCpy "$PROGRAMDIRECTORY" "$EXEDIR\$0"
		ReadINIStr $0 "$INIPATH\${NAME}.ini" "${NAME}" "SettingsDirectory"
		StrCpy "$SETTINGSDIRECTORY" "$EXEDIR\$0"
		ReadINIStr $0 "$INIPATH\${NAME}.ini" "${NAME}" "LAMEDirectory"
		StrCpy "$LAMEDIRECTORY" "$EXEDIR\$0"

		;=== Check that the above required parameters are present
		IfErrors NoINI

		ReadINIStr $0 "$INIPATH\${NAME}.ini" "${NAME}" "AdditionalParameters"
		StrCpy "$ADDITIONALPARAMETERS" $0
		ReadINIStr $0 "$INIPATH\${NAME}.ini" "${NAME}" "${APP}Executable"
		StrCpy "$PROGRAMEXECUTABLE" $0
		ReadINIStr $0 "$INIPATH\${NAME}.ini" "${NAME}" "DisableSplashScreen"
		StrCpy "$DISABLESPLASHSCREEN" $0

	;CleanUpAnyErrors:
		;=== Any missing unrequired INI entries will be an empty string, ignore associated errors
		ClearErrors

		;=== Correct PROGRAMEXECUTABLE if blank
		StrCmp $PROGRAMEXECUTABLE "" "" EndINI
			StrCpy "$PROGRAMEXECUTABLE" "${DEFAULTEXE}"
			Goto EndINI

	NoINI:
		;=== No INI file, so we'll use the defaults
		StrCpy "$ADDITIONALPARAMETERS" ""
		StrCpy "$PROGRAMEXECUTABLE" "${DEFAULTEXE}"
		StrCpy "$DISABLESPLASHSCREEN" "false"

		IfFileExists "$EXEDIR\App\${DEFAULTAPPDIR}\${DEFAULTEXE}" "" CheckPortableProgramDIR
			StrCpy "$PROGRAMDIRECTORY" "$EXEDIR\App\${DEFAULTAPPDIR}"
			StrCpy "$LAMEDIRECTORY" "$EXEDIR\App\LAME"
			StrCpy "$SETTINGSDIRECTORY" "$EXEDIR\Data\${DEFAULTSETTINGSPATH}"
			StrCpy "$ISDEFAULTDIRECTORY" "true"
			GoTo EndINI

		CheckPortableProgramDIR:
			IfFileExists "$EXEDIR\${NAME}\App\${DEFAULTAPPDIR}\${DEFAULTEXE}" "" NoProgramEXE
			StrCpy "$PROGRAMDIRECTORY" "$EXEDIR\${NAME}\App\${DEFAULTAPPDIR}"
			StrCpy "$LAMEDIRECTORY" "$EXEDIR\${NAME}\App\Lame"
			StrCpy "$SETTINGSDIRECTORY" "$EXEDIR\${NAME}\Data\${DEFAULTSETTINGSPATH}"
			GoTo EndINI
	
	EndINI:
		IfFileExists "$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" FoundProgramEXE

	NoProgramEXE:
		;=== Program executable not where expected
		MessageBox MB_OK|MB_ICONEXCLAMATION `$PROGRAMEXECUTABLE was not found.  Please check your configuration`
		Abort
		
	FoundProgramEXE:
		;=== Check if already running
		FindProcDLL::FindProc "$PROGRAMEXECUTABLE"                 
		StrCmp $R0 "1" "" CheckForSettings
			MessageBox MB_OK|MB_ICONINFORMATION `Another instance of ${APP} is already running.  Please close other instances of ${APP} before launching ${FULLNAME}.`
			Abort
	
	CheckForSettings:
		IfFileExists "$SETTINGSDIRECTORY\*.*" SettingsFound
		;=== No settings found
		StrCmp $ISDEFAULTDIRECTORY "true" CopyDefaultSettings
		CreateDirectory $SETTINGSDIRECTORY
		Goto SettingsFound
	
	CopyDefaultSettings:
		CreateDirectory "$EXEDIR\Data"
		CreateDirectory "$EXEDIR\Data\settings"
		CopyFiles /SILENT $EXEDIR\App\DefaultData\settings\*.* $EXEDIR\Data\settings
		GoTo SettingsFound

	SettingsFound:
		;=== Figure out the temp registry store
		StrCpy "$REGISTRYDIRECTORY" "$TEMP\${NAME}\registry"
		StrCpy "$APPTEMPDIRECTORY" "$TEMP\${NAME}\temp_files"
		CreateDirectory "$TEMP\${NAME}\temp_files"
		;SetOutPath $SETTINGSDIRECTORY

	;DisplaySplash:
		StrCmp $DISABLESPLASHSCREEN "true" GetPassedParameters
			;=== Show the splash screen before processing the files
			InitPluginsDir
			File /oname=$PLUGINSDIR\splash.jpg "${NAME}.jpg"	
			newadvsplash::show /NOUNLOAD 1500 200 0 -1 /L $PLUGINSDIR\splash.jpg

	GetPassedParameters:
		;=== Get any passed parameters
		Call GetParameters
		Pop $0
		StrCmp "'$0'" "''" "" LaunchProgramParameters

		;=== No parameters
		StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE"`
		Goto AdditionalParameters

	LaunchProgramParameters:
		StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" $0`

	AdditionalParameters:
		StrCmp $ADDITIONALPARAMETERS "" LaunchNow

		;=== Additional Parameters
		StrCpy $EXECSTRING `$EXECSTRING $ADDITIONALPARAMETERS`

	LaunchNow:
		CreateDirectory "$REGISTRYDIRECTORY"
		${registry::SaveKey} "HKEY_CURRENT_USER\Software\Audacity" "$REGISTRYDIRECTORY\audacity_local.reg" "" $0
		Sleep 100
		${registry::DeleteKey} "HKEY_CURRENT_USER\Software\Audacity" $0
		Sleep 100
		IfFileExists "$SETTINGSDIRECTORY\audacity_portable.reg" "" CreateOurKeys
		${registry::RestoreKey} "$SETTINGSDIRECTORY\audacity_portable.reg" $0
		Sleep 500
		
	CreateOurKeys:
		${registry::Write} "HKEY_CURRENT_USER\Software\Audacity\Audacity\Directories" "TempDir" "$APPTEMPDIRECTORY" "REG_SZ" $0
		Sleep 100
		${registry::Write} "HKEY_CURRENT_USER\Software\Audacity\Audacity" "WantAssociateFiles" 0 "REG_DWORD" $0
		Sleep 100
		IfFileExists "$LAMEDIRECTORY\lame_enc.dll" "" NoLAMEEncoder
			${registry::Write} "HKEY_CURRENT_USER\Software\Audacity\Audacity\MP3" "MP3LibPath" "$LAMEDIRECTORY\lame_enc.dll" "REG_SZ" $0
			Goto StartProgramNow
			
	NoLAMEEncoder:
		${registry::DeleteKey} "HKEY_CURRENT_USER\Software\Audacity\Audacity\MP3" $0

	StartProgramNow:
		IfFileExists "$TEMP\audacity_1_2_temp\*.*" ExecTheApp
			StrCpy $TEMPDIREXISTS 'true'
		
	ExecTheApp:
		ExecWait $EXECSTRING
	
	;TheEnd:
		;=== Restore registry keys
		${registry::SaveKey} "HKEY_CURRENT_USER\Software\Audacity" "$SETTINGSDIRECTORY\audacity_portable.reg" "" $0
		Sleep 100
		${registry::DeleteKey} "HKEY_CURRENT_USER\Software\Audacity" $0
		Sleep 100
		${registry::RestoreKey} "$REGISTRYDIRECTORY\audacity_local.reg" $0
	
	;EndWithoutRegRestore:
		StrCmp $DISABLESPLASHSCREEN "true" CleanupRunLocally
			Sleep 2000
			newadvsplash::stop

	CleanupRunLocally:
		${registry::Unload}
		RMDir "$TEMP\${NAME}\temp_files\"
		IfFileExists "$TEMP\${NAME}\temp_files\*.*" "" RemoveTempDir
		MessageBox MB_YESNO|MB_ICONQUESTION `The ${FULLNAME} temp directory still has files within it.  If ${FULLNAME} had a problem and shut down unexpectedly, you may be able to recover your work from these files if you start it again.  If ${FULLNAME} closed normally, you may safely delete them.  Would you like to delete these files?` IDYES RemoveTempDir
		RMDir /r $REGISTRYDIRECTORY
		Goto EndOfLauncher
		
	RemoveTempDir:
		RMDir /r "$TEMP\${NAME}\"
	
	EndOfLauncher:
		${registry::Unload}
		newadvsplash::stop
		StrCmp $TEMPDIREXISTS 'true' "" TheEnd
			RMDir '$TEMP\audacity_1_2_temp'

	TheEnd:
SectionEnd